@@ -1,17 +1,31 @@
+from __future__ import annotations
+
+import inspect
+from typing import TYPE_CHECKING, Any, cast
+
 import sqlalchemy as sa
 from marshmallow.fields import Field
-from marshmallow.schema import Schema, SchemaMeta, SchemaOpts
-from sqlalchemy.ext.declarative import DeclarativeMeta
+from marshmallow.schema import Schema, SchemaMeta, SchemaOpts, _get_fields
 
 from .convert import ModelConverter
 from .exceptions import IncorrectSchemaTypeError
-from .load_instance_mixin import LoadInstanceMixin
+from .load_instance_mixin import LoadInstanceMixin, _ModelType
+
+if TYPE_CHECKING:
+    from sqlalchemy.ext.declarative import DeclarativeMeta
 
 
 # This isn't really a field; it's a placeholder for the metaclass.
 # This should be considered private API.
 class SQLAlchemyAutoField(Field):
-    def __init__(self, *, column_name=None, model=None, table=None, field_kwargs):
+    def __init__(
+        self,
+        *,
+        column_name: str | None = None,
+        model: type[DeclarativeMeta] | None = None,
+        table: sa.Table | None = None,
+        field_kwargs: dict[str, Any],
+    ):
         super().__init__()
 
         if model and table:
@@ -22,20 +36,24 @@ class SQLAlchemyAutoField(Field):
         self.table = table
         self.field_kwargs = field_kwargs
 
-    def create_field(self, schema_opts, column_name, converter):
+    def create_field(
+        self,
+        schema_opts: SQLAlchemySchemaOpts,
+        column_name: str,
+        converter: ModelConverter,
+    ):
         model = self.model or schema_opts.model
         if model:
             return converter.field_for(model, column_name, **self.field_kwargs)
-        else:
-            table = self.table if self.table is not None else schema_opts.table
-            column = getattr(table.columns, column_name)
-            return converter.column2field(column, **self.field_kwargs)
+        table = self.table if self.table is not None else schema_opts.table
+        column = getattr(cast(sa.Table, table).columns, column_name)
+        return converter.column2field(column, **self.field_kwargs)
 
     # This field should never be bound to a schema.
     # If this method is called, it's probably because the schema is not a SQLAlchemySchema.
-    def _bind_to_schema(self, field_name, schema):
+    def _bind_to_schema(self, field_name: str, parent: Schema | Field) -> None:
         raise IncorrectSchemaTypeError(
-            f"Cannot bind SQLAlchemyAutoField. Make sure that {schema} is a SQLAlchemySchema or SQLAlchemyAutoSchema."
+            f"Cannot bind SQLAlchemyAutoField. Make sure that {parent} is a SQLAlchemySchema or SQLAlchemyAutoSchema."
         )
 
 
@@ -53,10 +71,12 @@ class SQLAlchemySchemaOpts(LoadInstanceMixin.Opts, SchemaOpts):
     - ``model_converter``: `ModelConverter` class to use for converting the SQLAlchemy model to marshmallow fields.
     """
 
+    table: sa.Table | None
+    model_converter: type[ModelConverter]
+
     def __init__(self, meta, *args, **kwargs):
         super().__init__(meta, *args, **kwargs)
 
-        self.model = getattr(meta, "model", None)
         self.table = getattr(meta, "table", None)
         if self.model is not None and self.table is not None:
             raise ValueError("Cannot set both `model` and `table` options.")
@@ -71,6 +91,9 @@ class SQLAlchemyAutoSchemaOpts(SQLAlchemySchemaOpts):
     - ``include_relationships``: Whether to include relationships; defaults to `False`.
     """
 
+    include_fk: bool
+    include_relationships: bool
+
     def __init__(self, meta, *args, **kwargs):
         super().__init__(meta, *args, **kwargs)
         self.include_fk = getattr(meta, "include_fk", False)
@@ -81,23 +104,46 @@ class SQLAlchemyAutoSchemaOpts(SQLAlchemySchemaOpts):
 
 class SQLAlchemySchemaMeta(SchemaMeta):
     @classmethod
-    def get_declared_fields(mcs, klass, cls_fields, inherited_fields, dict_cls):
+    def get_declared_fields(
+        mcs,
+        klass,
+        cls_fields: list[tuple[str, Field]],
+        inherited_fields: list[tuple[str, Field]],
+        dict_cls: type[dict] = dict,
+    ) -> dict[str, Field]:
         opts = klass.opts
-        Converter = opts.model_converter
+        Converter: type[ModelConverter] = opts.model_converter
         converter = Converter(schema_cls=klass)
         fields = super().get_declared_fields(
-            klass, cls_fields, inherited_fields, dict_cls
+            klass,
+            cls_fields,
+            # Filter out fields generated from foreign key columns
+            # if include_fk is set to False in the options
+            mcs._maybe_filter_foreign_keys(inherited_fields, opts=opts, klass=klass),
+            dict_cls,
         )
         fields.update(mcs.get_declared_sqla_fields(fields, converter, opts, dict_cls))
         fields.update(mcs.get_auto_fields(fields, converter, opts, dict_cls))
         return fields
 
     @classmethod
-    def get_declared_sqla_fields(mcs, base_fields, converter, opts, dict_cls):
+    def get_declared_sqla_fields(
+        mcs,
+        base_fields: dict[str, Field],
+        converter: ModelConverter,
+        opts: Any,
+        dict_cls: type[dict],
+    ) -> dict[str, Field]:
         return {}
 
     @classmethod
-    def get_auto_fields(mcs, fields, converter, opts, dict_cls):
+    def get_auto_fields(
+        mcs,
+        fields: dict[str, Field],
+        converter: ModelConverter,
+        opts: Any,
+        dict_cls: type[dict],
+    ) -> dict[str, Field]:
         return dict_cls(
             {
                 field_name: field.create_field(
@@ -109,10 +155,54 @@ class SQLAlchemySchemaMeta(SchemaMeta):
             }
         )
 
+    @staticmethod
+    def _maybe_filter_foreign_keys(
+        fields: list[tuple[str, Field]],
+        *,
+        opts: SQLAlchemySchemaOpts,
+        klass: SchemaMeta,
+    ) -> list[tuple[str, Field]]:
+        if opts.model is not None or opts.table is not None:
+            if not hasattr(opts, "include_fk") or opts.include_fk is True:
+                return fields
+            foreign_keys = {
+                column.key
+                for column in sa.inspect(opts.model or opts.table).columns  # type: ignore[union-attr]
+                if column.foreign_keys
+            }
+
+            non_auto_schema_bases = [
+                base
+                for base in inspect.getmro(klass)
+                if issubclass(base, Schema)
+                and not issubclass(base, SQLAlchemyAutoSchema)
+            ]
+
+            def is_declared_field(field: str) -> bool:
+                return any(
+                    field
+                    in [
+                        name
+                        for name, _ in _get_fields(
+                            getattr(base, "_declared_fields", base.__dict__)
+                        )
+                    ]
+                    for base in non_auto_schema_bases
+                )
+
+            return [
+                (name, field)
+                for name, field in fields
+                if name not in foreign_keys or is_declared_field(name)
+            ]
+        return fields
+
 
 class SQLAlchemyAutoSchemaMeta(SQLAlchemySchemaMeta):
     @classmethod
-    def get_declared_sqla_fields(cls, base_fields, converter, opts, dict_cls):
+    def get_declared_sqla_fields(
+        cls, base_fields, converter: ModelConverter, opts, dict_cls
+    ):
         fields = dict_cls()
         if opts.table is not None:
             fields.update(
@@ -141,7 +231,7 @@ class SQLAlchemyAutoSchemaMeta(SQLAlchemySchemaMeta):
 
 
 class SQLAlchemySchema(
-    LoadInstanceMixin.Schema, Schema, metaclass=SQLAlchemySchemaMeta
+    LoadInstanceMixin.Schema[_ModelType], Schema, metaclass=SQLAlchemySchemaMeta
 ):
     """Schema for a SQLAlchemy model or table.
     Use together with `auto_field` to generate fields from columns.
@@ -165,7 +255,9 @@ class SQLAlchemySchema(
     OPTIONS_CLASS = SQLAlchemySchemaOpts
 
 
-class SQLAlchemyAutoSchema(SQLAlchemySchema, metaclass=SQLAlchemyAutoSchemaMeta):
+class SQLAlchemyAutoSchema(
+    SQLAlchemySchema[_ModelType], metaclass=SQLAlchemyAutoSchemaMeta
+):
     """Schema that automatically generates fields from the columns of
      a SQLAlchemy model or table.
 
@@ -189,12 +281,13 @@ class SQLAlchemyAutoSchema(SQLAlchemySchema, metaclass=SQLAlchemyAutoSchemaMeta)
 
 
 def auto_field(
-    column_name: str = None,
+    column_name: str | None = None,
     *,
-    model: DeclarativeMeta = None,
-    table: sa.Table = None,
+    model: type[DeclarativeMeta] | None = None,
+    table: sa.Table | None = None,
+    # TODO: add type annotations for **kwargs
     **kwargs,
-):
+) -> SQLAlchemyAutoField:
     """Mark a field to autogenerate from a model or table.
 
     :param column_name: Name of the column to generate the field from.
